@@ -10,6 +10,7 @@
 import os
 import sys
 import argparse
+import argcomplete
 import logging
 import kaptan
 from . import config
@@ -23,6 +24,55 @@ logger = logging.getLogger(__name__)
 
 config_dir = os.path.expanduser('~/.tmuxp/')
 cwd_dir = os.getcwd() + '/'
+tmuxinator_config_dir = os.path.expanduser('~/.tmuxinator/')
+teamocil_config_dir = os.path.expanduser('~/.teamocil/')
+
+
+class ConfigCompleter(argcomplete.completers.FilesCompleter):
+
+    def __call__(self, prefix, **kwargs):
+        completion = argcomplete.completers.FilesCompleter.__call__(
+            self, prefix, **kwargs
+        )
+
+        completion += [os.path.join(config_dir, c)
+                       for c in config.in_dir(config_dir)]
+
+        return completion
+
+
+class TmuxinatorCompleter(argcomplete.completers.FilesCompleter):
+
+    def __call__(self, prefix, **kwargs):
+        completion = argcomplete.completers.FilesCompleter.__call__(
+            self, prefix, **kwargs
+        )
+
+        tmuxinator_configs = config.in_dir(
+            tmuxinator_config_dir, extensions='yml')
+        completion += [os.path.join(tmuxinator_config_dir, f)
+                       for f in tmuxinator_configs]
+
+        return completion
+
+
+class TeamocilCompleter(argcomplete.completers.FilesCompleter):
+
+    def __call__(self, prefix, **kwargs):
+        completion = argcomplete.completers.FilesCompleter.__call__(
+            self, prefix, **kwargs
+        )
+
+        teamocil_configs = config.in_dir(teamocil_config_dir, extensions='yml')
+        completion += [os.path.join(teamocil_config_dir, f)
+                       for f in teamocil_configs]
+
+        return completion
+
+
+def SessionCompleter(prefix, **kwargs):
+    t = Server()
+    return [s.get('session_name') for s in t._sessions if s.get('session_name').startswith(prefix)]
 
 
 def query_yes_no(question, default="yes"):
@@ -138,7 +188,7 @@ def build_workspace(config_file, args):
 
 
 def subcommand_load(args):
-    if args.list_configs:
+    if args.list:
         startup(config_dir)
         configs_in_user = config.in_dir(config_dir)
         configs_in_cwd = config.in_cwd()
@@ -159,36 +209,121 @@ def subcommand_load(args):
 
         print(output)
 
-    elif args.configs:
-        if '.' in args.configs:
-            args.configs.remove('.')
+    elif args.config:
+        if '.' == args.config:
             if config.in_cwd():
-                args.configs.append(config.in_cwd()[0])
+                args.config.append(config.in_cwd()[0])
             else:
                 print('No tmuxp configs found in current directory.')
 
-        for configfile in args.configs:
-            file_user = os.path.join(config_dir, configfile)
-            file_cwd = os.path.join(cwd_dir, configfile)
-            if os.path.exists(file_cwd) and os.path.isfile(file_cwd):
-                build_workspace(file_cwd, args)
-            elif os.path.exists(file_user) and os.path.isfile(file_user):
-                build_workspace(file_user, args)
+        configfile = args.config
+        file_user = os.path.join(config_dir, configfile)
+        file_cwd = os.path.join(cwd_dir, configfile)
+        if os.path.exists(file_cwd) and os.path.isfile(file_cwd):
+            build_workspace(file_cwd, args)
+        elif os.path.exists(file_user) and os.path.isfile(file_user):
+            build_workspace(file_user, args)
+        else:
+            logger.error('%s not found.' % configfile)
+
+
+def subcommand_import_teamocil(args):
+    if args.list:
+            try:
+                configs_in_user = config.in_dir(
+                    teamocil_config_dir, extensions='yml')
+            except OSError:
+                configs_in_user = []
+            configs_in_cwd = config.in_dir(
+                config_dir=cwd_dir, extensions='yml')
+
+            output = ''
+
+            if not os.path.exists(teamocil_config_dir):
+                output += '# %s: \n\tDirectory doesn\'t exist.\n' % teamocil_config_dir
+            elif not configs_in_user:
+                output += '# %s: \n\tNone found.\n' % teamocil_config_dir
             else:
-                logger.error('%s not found.' % configfile)
+                output += '# %s: \n\t%s\n' % (
+                    config_dir, ', '.join(configs_in_user)
+                )
+
+            if configs_in_cwd:
+                output += '# current directory:\n\t%s' % (
+                    ', '.join(configs_in_cwd)
+                )
+
+            print(output)
+
+    if args.config:
+        configfile = os.path.relpath(args.config)
+        configparser = kaptan.Kaptan(handler='yaml')
+        configparser.import_config(configfile)
+
+        newconfig = config.import_teamocil(configparser.get())
+
+        newconfig = configparser.import_config(newconfig)
+        newconfig = configparser.export(
+            'yaml', indent=2, default_flow_style=False
+        )
+
+        print(newconfig)
+
+
+def subcommand_import_tmuxinator(args):
+    if args.list:
+            try:
+                configs_in_user = config.in_dir(
+                    tmuxinator_config_dir, extensions='yml')
+            except OSError:
+                configs_in_user = []
+            configs_in_cwd = config.in_dir(
+                config_dir=cwd_dir, extensions='yml')
+
+            output = ''
+
+            if not os.path.exists(tmuxinator_config_dir):
+                output += '# %s: \n\tDirectory doesn\'t exist.\n' % tmuxinator_config_dir
+            elif not configs_in_user:
+                output += '# %s: \n\tNone found.\n' % tmuxinator_config_dir
+            else:
+                output += '# %s: \n\t%s\n' % (
+                    config_dir, ', '.join(configs_in_user)
+                )
+
+            if configs_in_cwd:
+                output += '# current directory:\n\t%s' % (
+                    ', '.join(configs_in_cwd)
+                )
+
+            print(output)
+
+    if args.config:
+        configfile = os.path.relpath(args.config)
+        configparser = kaptan.Kaptan(handler='yaml')
+        configparser.import_config(configfile)
+
+        newconfig = config.import_tmuxinator(configparser.get())
+
+        newconfig = configparser.import_config(newconfig)
+        newconfig = configparser.export(
+            'yaml', indent=2, default_flow_style=False
+        )
+
+        print(newconfig)
 
 
 def subcommand_convert(args):
-    if args.configs:
-        if '.' in args.configs:
-            args.configs.remove('.')
+    if args.config:
+        if '.' in args.config:
+            args.config.remove('.')
             if config.in_cwd():
-                args.configs.append(config.in_cwd()[0])
+                args.config.append(config.in_cwd()[0])
             else:
                 print('No tmuxp configs found in current directory.')
 
         try:
-            configfile = args.configs[0]
+            configfile = args.config
         except Exception:
             print('Please enter a config')
 
@@ -234,17 +369,17 @@ def subcommand_convert(args):
 
 def subcommand_attach_session(args):
     commands = []
-    try:
-        ctext = args.session_name[0]
-    except IndexError as e:
-        return
+    ctext = args.session_name
 
-    t = Server()
+    t = Server(
+        socket_name=args.socket_name or None,
+        socket_path=args.socket_path or None
+    )
     try:
-        session = [s for s in t.sessions if s.get('session_name') == ctext][0]
-    except IndexError as e:
-        print('Session not found.')
-        return
+        session = next((s for s in t.sessions if s.get(
+            'session_name') == ctext), None)
+        if not session:
+            raise Exception('Session not found.')
     except Exception as e:
         print(e.message[0])
         return
@@ -260,20 +395,26 @@ def subcommand_attach_session(args):
 
 def subcommand_kill_session(args):
     commands = []
-    ctext = args.session_name[0]
+    ctext = args.session_name
+
+    t = Server(
+        socket_name=args.socket_name or None,
+        socket_path=args.socket_path or None
+    )
 
-    t = Server()
     try:
-        sessions = [s for s in t.sessions if s.get('session_name') == ctext]
+        session = next((s for s in t.sessions if s.get(
+            'session_name') == ctext), None)
+        if not session:
+            raise Exception('Session not found.')
     except Exception as e:
         print(e.message[0])
         return
 
-    if (len(sessions) == 1):
-        try:
-            sessions[0].kill_session()
-        except Exception as e:
-            logger.error(e)
+    try:
+        session.kill_session()
+    except Exception as e:
+        logger.error(e)
 
 
 def cli_parser():
@@ -284,7 +425,6 @@ def cli_parser():
         ''',
     )
 
-    parser = argparse.ArgumentParser()
     subparsers = parser.add_subparsers(title='subcommands',
                                        description='valid subcommands',
                                        help='additional help')
@@ -294,32 +434,30 @@ def cli_parser():
 
     kill_session.add_argument(
         dest='session_name',
-        nargs='*',
         type=str,
         default=None,
-    )
+    ).completer = SessionCompleter
 
     attach_session = subparsers.add_parser('attach-session')
     attach_session.set_defaults(callback=subcommand_attach_session)
 
     attach_session.add_argument(
         dest='session_name',
-        nargs='*',
         type=str,
-        default=None,
-    )
+    ).completer = SessionCompleter
 
     load = subparsers.add_parser('load')
 
-    load.add_argument(
-        '-l', '--list', dest='list_configs', action='store_true',
-        help='List config files available')
+    loadgroup = load.add_mutually_exclusive_group(required=True)
+    loadgroup.add_argument(
+        '--list', dest='list', action='store_true',
+        help='List config files available',
+    )
 
-    load.add_argument(
-        dest='configs',
-        nargs='*',
+    loadgroup.add_argument(
+        dest='config',
         type=str,
-        default=None,
+        nargs='?',
         help='''\
         List of config files to launch session from.
 
@@ -329,15 +467,14 @@ def cli_parser():
 
         will check launch a ~/.pullv.yaml / ~/.pullv.json from the cwd.
         will also check for any ./*.yaml and ./*.json.
-        ''' % (cwd_dir + '/', config_dir)
-    )
+        ''' % (cwd_dir + '/', config_dir),
+    ).completer = ConfigCompleter(allowednames=('.yaml', '.json'), directories=False)
     load.set_defaults(callback=subcommand_load)
 
     convert = subparsers.add_parser('convert')
 
     convert.add_argument(
-        dest='configs',
-        nargs='*',
+        dest='config',
         type=str,
         default=None,
         help='''\
@@ -348,9 +485,54 @@ def cli_parser():
         will check launch a ~/.pullv.yaml / ~/.pullv.json from the cwd.
         will also check for any ./*.yaml and ./*.json.
         ''' % (cwd_dir + '/', config_dir)
-    )
+    ).completer = ConfigCompleter(allowednames=('.yaml', '.json'), directories=False)
+
     convert.set_defaults(callback=subcommand_convert)
 
+    importparser = subparsers.add_parser('import')
+    importsubparser = importparser.add_subparsers(title='subcommands',
+                                                  description='valid subcommands',
+                                                  help='additional help')
+
+    import_teamocil = importsubparser.add_parser('teamocil')
+
+    import_teamocilgroup = import_teamocil.add_mutually_exclusive_group(
+        required=True)
+    import_teamocilgroup.add_argument(
+        '--list', dest='list', action='store_true',
+        help='List yaml configs in ~/.teamocil and current working directory.'
+    )
+
+    import_teamocilgroup.add_argument(
+        dest='config',
+        type=str,
+        nargs='?',
+        help='''\
+        Checks current ~/.teamocil and current directory for yaml files.
+        '''
+    ).completer = TeamocilCompleter(allowednames=('.yml'), directories=False)
+    import_teamocil.set_defaults(callback=subcommand_import_teamocil)
+
+    import_tmuxinator = importsubparser.add_parser('tmuxinator')
+
+    import_tmuxinatorgroup = import_tmuxinator.add_mutually_exclusive_group(
+        required=True)
+    import_tmuxinatorgroup.add_argument(
+        '--list', dest='list', action='store_true',
+        help='List yaml configs in ~/.tmuxinator and current working directory.'
+    )
+
+    import_tmuxinatorgroup.add_argument(
+        dest='config',
+        type=str,
+        nargs='?',
+        help='''\
+        Checks current ~/.tmuxinator and current directory for yaml files.
+        '''
+    ).completer = TmuxinatorCompleter(allowednames=('.yml'), directories=False)
+
+    import_tmuxinator.set_defaults(callback=subcommand_import_tmuxinator)
+
     parser.add_argument('--log-level', dest='log_level', default='INFO',
                         metavar='log-level',
                         help='Log level e.g. INFO, DEBUG, ERROR')
@@ -374,6 +556,8 @@ def main():
 
     parser = cli_parser()
 
+    argcomplete.autocomplete(parser, always_complete_options=False)
+
     args = parser.parse_args()
 
     setupLogger(level=args.log_level.upper())
@@ -390,75 +574,13 @@ def main():
         subcommand_load(args)
     elif args.callback is subcommand_convert:
         subcommand_convert(args)
+    elif args.callback is subcommand_import_teamocil:
+        subcommand_import_teamocil(args)
+    elif args.callback is subcommand_import_tmuxinator:
+        subcommand_import_tmuxinator(args)
     elif args.callback is subcommand_attach_session:
         subcommand_attach_session(args)
     elif args.callback is subcommand_kill_session:
         subcommand_kill_session(args)
     else:
         parser.print_help()
-
-
-def complete(cline, cpoint):
-
-    parser = argparse.ArgumentParser()
-    parser.add_argument('-L', dest='socket_name', default=None,
-                        metavar='socket-name')
-
-    parser.add_argument('-S', dest='socket_path', default=None,
-                        metavar='socket-path')
-
-    parser.add_argument(
-        dest='ctexta',
-        nargs='*',
-        type=str,
-        default=None,
-    )
-
-    args = parser.parse_args()
-    ctext = cline.replace('tmuxp ', '')
-
-    commands = []
-    commands.extend(['attach-session', 'kill-session', 'load', 'convert'])
-
-    commands = [c for c in commands if ctext in c]
-
-    if args.socket_path:
-        ctext = ctext.replace(args.socket_path or None, '')
-
-    if args.socket_name:
-        ctext = ctext.replace(args.socket_name or None, '')
-
-    t = Server(
-        socket_name=args.socket_name or None,
-        socket_path=args.socket_path or None
-    )
-
-    def session_complete(command, commands, ctext):
-        if ctext.startswith(command + ' '):
-            commands[:] = []
-            ctext_subargs = ctext.replace(command + ' ', '')
-
-            try:
-                sessions = [s.get('session_name') for s in t._sessions]
-                commands.extend([c for c in sessions if ctext_subargs in c])
-            except Exception:
-                pass
-
-    def config_complete(command, commands, ctext):
-        if ctext.startswith(command + ' '):
-            commands[:] = []
-            ctext_subargs = ctext.replace(command + ' ', '')
-            configs = []
-
-            configs += ['./' + c for c in config.in_dir(cwd_dir)]
-            configs += ['./' + c for c in config.in_cwd()]
-            configs += [os.path.join(config_dir, c)
-                        for c in config.in_dir(config_dir)]
-            commands += [c for c in configs if c.startswith(ctext_subargs)]
-
-    session_complete('attach-session', commands, ctext)
-    session_complete('kill-session', commands, ctext)
-    config_complete('load', commands, ctext)
-    config_complete('convert', commands, ctext)
-
-    print(' \n'.join(commands))
